@@ -31,16 +31,21 @@ class Debug:
     # Enable debug output from specific classes only
     Whitelist = [
         "About_Dialog",
+        "API",
         "Assert_Dialog",
+        "Backend_Types",
         "BiotSavart_CUDA",
         "BiotSavart_JIT",
         "CalculationThread",
+        "CheckForUpdates_Dialog",
         # "Config",
         "Constants",
         "Constraint",
         "Constraint_Editor",
         "Display_Widget",
+        "ExportContainer_Dialog",
         "Field",
+        "Field_Types",
         "Field_Widget",
         "Groupbox",
         "GUI",
@@ -52,10 +57,12 @@ class Debug:
         "Metric_Widget",
         "Model",
         "ModelAccess",
+        "OverridePadding_Dialog",
         "Parameters",
         "Parameters_Widget",
         "Perspective_Presets",
         "Perspective_Widget",
+        "QtWidgets2",
         "SamplingVolume",
         "SamplingVolume_Widget",
         "SidebarLeft",
