@@ -17,7 +17,11 @@
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 
-__VERSION__ = "v1.10.1"
+__VERSION__ = "v1.10.2"
+
+
+__URL__ = "https://github.com/shredEngineer/MagnetiCalc"
+__VERSION__URL__ = "https://raw.githubusercontent.com/shredEngineer/MagnetiCalc/master/magneticalc/Version.py"
 
 
 class Version:
